/* 
 *	HT Editor
 *	htcfg.cc
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "htapp.h"
#include "htcfg.h"
#include "htctrl.h"
#include "htdebug.h"
#include "htendian.h"
#include "htstring.h"
#include "stream.h"
#include "store.h"
#include "htsys.h"
#include "tools.h"

#include <errno.h>
#include <stdlib.h>
#include <string.h>

/* VERSION 2 (for ht-0.4.4 and later) */

/* NOTE: as of Version 2 ALL integers in HT config-files are
   stored in big-endian format... (non-intel) */
   
#define object_stream_bin	0
#define object_stream_txt	1

int def_project_ostream_type=object_stream_bin;

int def_file_ostream_type=object_stream_bin;

ht_object_stream *create_object_stream(ht_stream *f, int object_stream_type)
{
	ht_object_stream *s;
	switch (object_stream_type) {
		case object_stream_bin:
			s=new ht_object_stream_bin();
			((ht_object_stream_bin*)s)->init(f, true);
			break;
		case object_stream_txt:
			s=new ht_object_stream_txt();
			((ht_object_stream_txt*)s)->init(f);
			break;
		default:
			assert(0);
	}
	return s;
}

struct config_header {
	char magic[4] HTPACKED;
	char version[4] HTPACKED;
	char stream_type[2] HTPACKED;
};

/*
 *	project configs
 */

char *projectconfig_file;

#define ht_projectconfig_magic			"HTCP"
#define ht_projectconfig_fileversion		2

/**/

loadstore_result save_projectconfig()
{
	ht_file *f=new ht_file();
	f->init(projectconfig_file, FAM_WRITE | FAM_CREATE);
	if (f->get_error()) {
		f->done();
		delete f;
		return LS_ERROR_WRITE;
	}
	
/* write project config header */
	config_header h;

	memmove(h.magic, ht_projectconfig_magic, sizeof h.magic);

	char q[16];

	sprintf(q, "%04x", ht_projectconfig_fileversion);
	memmove(h.version, q, sizeof h.version);

	sprintf(q, "%02x", def_project_ostream_type);
	memmove(h.stream_type, q, sizeof h.stream_type);

	f->write(&h, sizeof h);
	
/* write object stream type */
	ht_object_stream *d=create_object_stream(f, def_project_ostream_type);
	   
/* write config */
	app->store(d);
	if (d->get_error()) return LS_ERROR_WRITE;
	
	f->done();
	delete f;
	
	d->done();
	delete d;
	
	return LS_OK;
}

loadstore_result load_projectconfig()
{
	ht_file *f=new ht_file();
	f->init(projectconfig_file, FAM_READ);
	switch (f->get_error()) {
		case 0:break;
		case STERR_SYSTEM | ENOENT:
			f->done();
			delete f;
			return LS_ERROR_NOT_FOUND;
		default:
			f->done();
			delete f;
			return LS_ERROR_READ;
	}
/* read project config header */
	config_header h;

	f->read(&h, sizeof h);
	
	if (memcmp(h.magic, ht_projectconfig_magic, sizeof h.magic)!=0) return LS_ERROR_MAGIC;

	if (hexw((char*)h.version)!=ht_projectconfig_fileversion) return LS_ERROR_VERSION;
	
	dword object_stream_type=hexb((char*)h.stream_type);

/* object stream type */
	ht_object_stream *d=create_object_stream(f, object_stream_type);
	   
/* read config */
	if (app->load(d)!=0) {
		if (d->get_error()) {
			if (object_stream_type==object_stream_txt)
				printf("error in line %d\n", ((ht_object_stream_txt*)d)->get_error_line());
		}
		return LS_ERROR_CORRUPTED;
	}
	
	f->done();
	delete f;
	
	d->done();
	delete d;
	
	return LS_OK;
}

/* file configs */

#define ht_fileconfig_magic				"HTCF"
#define ht_fileconfig_fileversion			2

/**/

loadstore_result save_fileconfig(char *fileconfig_file, ht_window *window)
{
	ht_file *f=new ht_file();
	f->init(fileconfig_file, FAM_WRITE | FAM_CREATE);
	if (f->get_error()) {
		f->done();
		delete f;
		return LS_ERROR_WRITE;
	}
	
/* write file config header */
	config_header h;

	memmove(h.magic, ht_fileconfig_magic, sizeof h.magic);

	char q[16];

	sprintf(q, "%04x", ht_fileconfig_fileversion);
	memmove(h.version, q, sizeof h.version);

	sprintf(q, "%02x", def_file_ostream_type);
	memmove(h.stream_type, q, sizeof h.stream_type);

	f->write(&h, sizeof h);

/* object stream type */
	ht_object_stream *d=create_object_stream(f, def_file_ostream_type);
	   
/* write config */
	d->put_object(window, NULL);

	f->done();
	delete f;
	
	d->done();
	delete d;
	
	return LS_OK;
}

ht_window *load_fileconfig(char *fileconfig_file)
{
	ht_file *f=new ht_file();
	f->init(fileconfig_file, FAM_READ);
	if (f->get_error()) {
		f->done();
		delete f;
		return 0;
	}		    
/* read file config header */
	config_header h;

	f->read(&h, sizeof h);
	
	if (memcmp(h.magic, ht_fileconfig_magic, sizeof h.magic)!=0) return 0;

	if (hexw((char*)h.version)!=ht_fileconfig_fileversion) return 0;
	
	dword object_stream_type=hexb((char*)h.stream_type);

/* object stream type */
	ht_object_stream *d=create_object_stream(f, object_stream_type);
	   
/* read config */
	ht_window *w=(ht_window*)d->get_object(NULL);

	f->done();
	delete f;
	
	d->done();
	delete d;
	
	return w;
}

/*
 *	INIT
 */

bool init_cfg()
{
#if defined(WIN32) || defined(__WIN32__) || defined(MSDOS) || defined(DJGPP)
	char d[1024];	/* FIXME: !!!! */
	sys_dirname(this_app, d);
	char *b="/"PROJECT_CONFIG_FILE_NAME;
	projectconfig_file=(char*)malloc(strlen(d)+strlen(b)+1);
	strcpy(projectconfig_file, d);
	strcat(projectconfig_file, b);
#else
	char *home=getenv("HOME");
	char *b="/"PROJECT_CONFIG_FILE_NAME;
	if (!home) home="";
	projectconfig_file=(char*)malloc(strlen(home)+strlen(b)+1);
	strcpy(projectconfig_file, home);
	strcat(projectconfig_file, b);
#endif
	return true;
}

/*
 *	DONE
 */

void done_cfg()
{
	free(projectconfig_file);
}

